use crate::{Context, Handler};
use actix_web::{post, web, HttpResponse, Responder};
use serde::{Deserialize, Serialize};
use serde_json::json;
use verax::{RevId, TxId};

#[derive(Deserialize)]
pub struct Lock {
    id: TxId,
    client_name: String,
}

#[derive(Serialize)]
pub struct Response {
    id: TxId,
    #[serde(rename = "_rev")]
    revision_id: RevId,
    secret: String,
}

#[async_trait::async_trait]
impl Handler for Lock {
    type Ok = Response;
    type Err = verax::Error;

    async fn handle(self, ledger: &Context) -> Result<Self::Ok, Self::Err> {
        let (new_tx, token) = ledger
            .ledger
            .lock_transaction(self.id, self.client_name)
            .await?;
        Ok(Response {
            id: new_tx.id,
            revision_id: new_tx.revision_id,
            secret: token.to_string(),
        })
    }
}

#[post("/lock")]
async fn handler(item: web::Json<Lock>, ctx: web::Data<Context>) -> impl Responder {
    match item.into_inner().handle(&ctx).await {
        Ok(tx) => HttpResponse::Accepted().json(tx),
        Err(err) => {
            HttpResponse::InternalServerError().json(json!({ "text": err.to_string(), "err": err}))
        }
    }
}
